require 'uri'
require 'net/http'
require 'active_support/core_ext'
require 'action_dispatch'
require 'appmonitor/event_notification'

module AppMonitor
  class EventNotifier

    API_MESSAGE_URL = 'http://team10-14.ucebne.fiit.stuba.sk/api/message'

    def initialize(app)
      @app = app
      configure
      @config[:ignore_exceptions] ||= self.class.default_ignore_exceptions
    end

    def configure(opts = {})
      @config = {
          api_key: '',
          project_id: '',
          monitoring: {development: '', staging: '', production: ''}
      }
      @valid_config_keys = @config.keys
      path_to_yaml_file = "#{Rails.root}/config/appmonitor.yml"
      begin
        config = YAML::load(IO.read(path_to_yaml_file))
        Rails.logger.info(config)
      rescue Errno::ENOENT
        return
      rescue Psych::SyntaxError
        return
      end
      config.each { |k, v| @config[k.to_sym] = v if @valid_config_keys.include? k.to_sym }
    end

    def self.default_ignore_exceptions
      [].tap do |exceptions|
        exceptions << 'ActionController::RoutingError'
        exceptions << 'ActionController::InvalidAuthenticityToken'
        exceptions << 'CGI::Session::CookieStore::TamperedWithCookie'
        exceptions << 'ActionController::UnknownAction'
        exceptions << 'ActionController::UnknownFormat'
        exceptions << 'ActionController::UnknownHttpMethod'
        exceptions << 'Mongoid::Errors::DocumentNotFound'
        exceptions << 'AbstractController::ActionNotFound'
      end
    end

    def ignored_exception(ignore_array, exception)
      Array.wrap(ignore_array).map(&:to_s).include?(exception.class.name)
    end

    def call(env)
      @app.call(env)
    rescue Exception => exception
      if @config[:monitoring]["#{Rails.env.to_s.downcase}"]
        @request = ActionDispatch::Request.new(env)
        send_exception(exception)
      end
      raise exception
    end

    def send_exception(exception)
      unless ignored_exception(@config[:ignore_exceptions], exception)
        event = EventNotification.build_exception_hash(exception, @request)
        @event = event.to_json
        notify_on_event("Error")
      end
    end

    def send_rake_event(exception, options = {})
      configure
      unless (ignored_exception(@config[:ignore_exceptions], exception) ||  !@config[:monitoring]["#{Rails.env.to_s.downcase}"])
        event = EventNotification.build_rake_event(exception, options)
        @event = event.to_json
        notify_on_event("RakeError")
      end
    end

    def send_custom_event(type, options={})
      @event = options.to_json
      notify_on_event(type)
    end

    def notify_on_event(type)
      uri = URI.parse(API_MESSAGE_URL)
      https = Net::HTTP.new(uri.host, uri.port)
      https.use_ssl = false
      request = Net::HTTP::Post.new(uri.path, initheader = {'Content-Type' => "application/json",
                                                            'appm_apiKey' => @config[:api_key],
                                                            'appm_projectId' => @config[:project_id],
                                                            'appm_type' => type}
      )

      request.body = @event
      response = https.request(request)

      Rails.logger.debug ("Appmonitor: Event has been sent")
      puts 'Appmonitor: Event has been sent'

      unless response.kind_of? Net::HTTPSuccess
        Rails.logger.warn('Appmonitor WARNING: Something went wrong, please check your config')
        Rails.logger.warn("AppMonitor WARNING: Response ''#{response.code}: #{response.message}''")
        puts 'Appmonitor WARNING: Something went wrong, please check your config'
        puts "AppMonitor WARNING: Response #{response.inspect}"
      end
    end

  end
end
